@@ -2,6 +2,7 @@
 
 /* 
  * Copyright (C) 2001 Havoc Pennington, Copyright (C) 2002 Red Hat Inc.
+ * Copyright (C) 2006 Elijah Newren
  * 
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -41,6 +42,7 @@
  */
 #define KEY_MOUSE_BUTTON_MODS "/apps/metacity/general/mouse_button_modifier"
 #define KEY_FOCUS_MODE "/apps/metacity/general/focus_mode"
+#define KEY_RAISE_ON_CLICK "/apps/metacity/general/raise_on_click"
 #define KEY_ACTION_DOUBLE_CLICK_TITLEBAR "/apps/metacity/general/action_double_click_titlebar"
 #define KEY_AUTO_RAISE "/apps/metacity/general/auto_raise"
 #define KEY_AUTO_RAISE_DELAY "/apps/metacity/general/auto_raise_delay"
@@ -52,6 +54,7 @@
 #define KEY_DISABLE_WORKAROUNDS "/apps/metacity/general/disable_workarounds"
 #define KEY_BUTTON_LAYOUT "/apps/metacity/general/button_layout"
 #define KEY_REDUCED_RESOURCES "/apps/metacity/general/reduced_resources"
+#define KEY_GNOME_ACCESSIBILITY "/desktop/gnome/interface/accessibility"
 
 #define KEY_COMMAND_PREFIX "/apps/metacity/keybinding_commands/command_"
 
@@ -65,6 +68,8 @@
 #define KEY_VISUAL_BELL "/apps/metacity/general/visual_bell"
 #define KEY_AUDIBLE_BELL "/apps/metacity/general/audible_bell"
 #define KEY_VISUAL_BELL_TYPE "/apps/metacity/general/visual_bell_type"
+#define KEY_CURSOR_THEME "/desktop/gnome/peripherals/mouse/cursor_theme"
+#define KEY_CURSOR_SIZE "/desktop/gnome/peripherals/mouse/cursor_size"
 
 #ifdef HAVE_GCONF
 static GConfClient *default_client = NULL;
@@ -77,6 +82,7 @@ static gboolean use_system_font = TRUE;
 static PangoFontDescription *titlebar_font = NULL;
 static MetaVirtualModifier mouse_button_mods = Mod1Mask;
 static MetaFocusMode focus_mode = META_FOCUS_MODE_CLICK;
+static gboolean raise_on_click = TRUE;
 static char* current_theme = NULL;
 static int num_workspaces = 4;
 static MetaActionDoubleClickTitlebar action_double_click_titlebar =
@@ -88,6 +94,9 @@ static gboolean auto_raise_delay = 500;
 static gboolean provide_visual_bell = TRUE;
 static gboolean bell_is_audible = TRUE;
 static gboolean reduced_resources = FALSE;
+static gboolean gnome_accessibility = FALSE;
+static char *cursor_theme = NULL;
+static int   cursor_size = 24;
 
 static MetaVisualBellType visual_bell_type = META_VISUAL_BELL_INVALID;
 static MetaButtonLayout button_layout = {
@@ -117,6 +126,7 @@ static gboolean update_use_system_font   (gboolean    value);
 static gboolean update_titlebar_font      (const char *value);
 static gboolean update_mouse_button_mods  (const char *value);
 static gboolean update_focus_mode         (const char *value);
+static gboolean update_raise_on_click     (gboolean    value);
 static gboolean update_theme              (const char *value);
 static gboolean update_visual_bell        (gboolean v1, gboolean v2);
 static gboolean update_visual_bell_type   (const char *value);
@@ -139,6 +149,9 @@ static gboolean update_terminal_command   (const char *value);
 static gboolean update_workspace_name     (const char  *name,
                                            const char  *value);
 static gboolean update_reduced_resources  (gboolean     value);
+static gboolean update_gnome_accessibility  (gboolean     value);
+static gboolean update_cursor_theme       (const char *value);
+static gboolean update_cursor_size        (int size);
 
 static void change_notify (GConfClient    *client,
                            guint           cnxn_id,
@@ -310,6 +323,21 @@ meta_prefs_init (void)
                         &err);
   cleanup_error (&err);
 
+  gconf_client_add_dir (default_client, "/desktop/gnome/applications/terminal",
+                        GCONF_CLIENT_PRELOAD_RECURSIVE,
+                        &err);
+  cleanup_error (&err);
+
+  gconf_client_add_dir (default_client, KEY_GNOME_ACCESSIBILITY,
+                        GCONF_CLIENT_PRELOAD_RECURSIVE,
+                        &err);
+  cleanup_error (&err);
+
+  gconf_client_add_dir (default_client, "/desktop/gnome/peripherals/mouse",
+                        GCONF_CLIENT_PRELOAD_RECURSIVE,
+                        &err);
+  cleanup_error (&err);
+
   str_val = gconf_client_get_string (default_client, KEY_MOUSE_BUTTON_MODS,
                                      &err);
   cleanup_error (&err);
@@ -322,6 +350,11 @@ meta_prefs_init (void)
   update_focus_mode (str_val);
   g_free (str_val);  
 
+  bool_val = gconf_client_get_bool (default_client, KEY_RAISE_ON_CLICK,
+				    &err);
+  cleanup_error (&err);
+  update_raise_on_click (bool_val);
+  
   str_val = gconf_client_get_string (default_client,
                                      KEY_ACTION_DOUBLE_CLICK_TITLEBAR,
 				     &err);
@@ -407,6 +440,23 @@ meta_prefs_init (void)
   cleanup_error (&err);
   update_terminal_command (str_val);
   g_free (str_val);
+
+  bool_val = gconf_client_get_bool (default_client, KEY_GNOME_ACCESSIBILITY,
+                                    &err);
+  
+  cleanup_error (&err);
+  update_gnome_accessibility (bool_val);
+
+  str_val = gconf_client_get_string (default_client, KEY_CURSOR_THEME,
+                                     &err);
+  cleanup_error (&err);
+  update_cursor_theme (str_val);
+  g_free (str_val);
+
+  int_val = gconf_client_get_int (default_client, KEY_CURSOR_SIZE,
+                                  &err);
+  cleanup_error (&err);
+  update_cursor_size (int_val);
 #endif /* HAVE_GCONF */
   
   /* Load keybindings prefs */
@@ -429,6 +479,17 @@ meta_prefs_init (void)
                            NULL,
                            NULL,
                            &err);
+  gconf_client_notify_add (default_client, KEY_GNOME_ACCESSIBILITY,
+                           change_notify,
+                           NULL,
+                           NULL,
+                           &err);
+  gconf_client_notify_add (default_client, "/desktop/gnome/peripherals/mouse",
+                           change_notify,
+                           NULL,
+                           NULL,
+                           &err);
+
   cleanup_error (&err);  
 #endif /* HAVE_GCONF */
 }
@@ -499,6 +560,22 @@ change_notify (GConfClient    *client,
       if (update_focus_mode (str))
         queue_changed (META_PREF_FOCUS_MODE);
     }
+  else if (strcmp (key, KEY_RAISE_ON_CLICK) == 0)
+    {
+      gboolean b;
+
+      if (value && value->type != GCONF_VALUE_BOOL)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                        KEY_RAISE_ON_CLICK);
+          goto out;
+        }
+
+      b = value ? gconf_value_get_bool (value) : TRUE;
+      
+      if (update_raise_on_click (b))
+        queue_changed (META_PREF_RAISE_ON_CLICK);
+    }
   else if (strcmp (key, KEY_THEME) == 0)
     {
       const char *str;
@@ -805,6 +882,54 @@ change_notify (GConfClient    *client,
       if (update_reduced_resources (b))
         queue_changed (META_PREF_REDUCED_RESOURCES);
     }
+  else if (strcmp (key, KEY_GNOME_ACCESSIBILITY) == 0)
+    {
+      gboolean b;
+
+      if (value && value->type != GCONF_VALUE_BOOL)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                       KEY_GNOME_ACCESSIBILITY);
+          goto out;
+        }
+
+      b = value ? gconf_value_get_bool (value) : gnome_accessibility;
+
+      if (update_gnome_accessibility (b))
+        queue_changed (META_PREF_GNOME_ACCESSIBILITY);
+    }
+  else if (strcmp (key, KEY_CURSOR_THEME) == 0)
+    {
+      const char *str;
+
+      if (value && value->type != GCONF_VALUE_STRING)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                       KEY_CURSOR_THEME);
+          goto out;
+        }
+
+      str = value ? gconf_value_get_string (value) : NULL;
+
+      if (update_cursor_theme (str))
+	queue_changed (META_PREF_CURSOR_THEME);
+    }
+  else if (strcmp (key, KEY_CURSOR_SIZE) == 0)
+    {
+      int d;
+
+      if (value && value->type != GCONF_VALUE_INT)
+        {
+          meta_warning (_("GConf key \"%s\" is set to an invalid type\n"),
+                       KEY_CURSOR_SIZE);
+          goto out;
+        }
+
+      d = value ? gconf_value_get_int (value) : 24;
+
+      if (update_cursor_size (d))
+	queue_changed (META_PREF_CURSOR_SIZE);
+    }
   else
     {
       meta_topic (META_DEBUG_PREFS, "Key %s doesn't mean anything to Metacity\n",
@@ -872,6 +997,18 @@ update_focus_mode (const char *value)
 }
 #endif /* HAVE_GCONF */
 
+#ifdef HAVE_GCONF
+static gboolean
+update_raise_on_click (gboolean value)
+{
+  gboolean old = raise_on_click;
+
+  raise_on_click = value;
+
+  return old != value;
+}
+#endif /* HAVE_GCONF */
+
 #ifdef HAVE_GCONF
 static gboolean
 update_theme (const char *value)
@@ -918,12 +1055,72 @@ meta_prefs_get_focus_mode (void)
   return focus_mode;
 }
 
+gboolean
+meta_prefs_get_raise_on_click (void)
+{
+  /* Force raise_on_click on for click-to-focus, as requested by Havoc
+   * in #326156.
+   */
+  return raise_on_click || focus_mode == META_FOCUS_MODE_CLICK;
+}
+
 const char*
 meta_prefs_get_theme (void)
 {
   return current_theme;
 }
 
+#ifdef HAVE_GCONF
+static gboolean
+update_cursor_theme (const char *value)
+{
+  char *old_theme;
+  gboolean changed;
+  
+  old_theme = cursor_theme;
+  
+  if (value != NULL && *value)
+    {
+      cursor_theme = g_strdup (value);
+    }
+
+  changed = TRUE;
+  if ((old_theme && cursor_theme &&
+       strcmp (old_theme, cursor_theme) == 0) ||
+      (old_theme == NULL && cursor_theme == NULL))
+    changed = FALSE;
+
+  if (old_theme != cursor_theme)
+    g_free (old_theme);
+
+  return changed;
+}
+#endif /* HAVE_GCONF */
+
+const char*
+meta_prefs_get_cursor_theme (void)
+{
+  return cursor_theme;
+}
+
+#ifdef HAVE_GCONF
+static gboolean
+update_cursor_size (int value)
+{
+  int old = cursor_size;
+
+  cursor_size = value;
+
+  return old != value;
+}
+#endif
+
+int
+meta_prefs_get_cursor_size (void)
+{
+  return cursor_size;
+}
+
 #ifdef HAVE_GCONF
 static gboolean
 update_use_system_font (gboolean value)
@@ -1321,6 +1518,16 @@ update_reduced_resources (gboolean value)
 
   return old != reduced_resources;
 }
+
+static gboolean
+update_gnome_accessibility (gboolean value)
+{
+  gboolean old = gnome_accessibility;
+
+  gnome_accessibility = value;
+  
+  return old != gnome_accessibility;
+}
 #endif /* HAVE_GCONF */
 
 #ifdef WITH_VERBOSE_MODE
@@ -1335,6 +1542,9 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_FOCUS_MODE:
       return "FOCUS_MODE";
 
+    case META_PREF_RAISE_ON_CLICK:
+      return "RAISE_ON_CLICK";
+      
     case META_PREF_THEME:
       return "THEME";
 
@@ -1394,6 +1604,15 @@ meta_preference_to_string (MetaPreference pref)
     case META_PREF_REDUCED_RESOURCES:
       return "REDUCED_RESOURCES";
       break;
+    case META_PREF_GNOME_ACCESSIBILITY:
+      return "GNOME_ACCESSIBILTY";
+      break;
+    case META_PREF_CURSOR_THEME:
+      return "CURSOR_THEME";
+      break;
+    case META_PREF_CURSOR_SIZE:
+      return "CURSOR_SIZE";
+      break;
     }
 
   return "(unknown)";
@@ -2107,6 +2326,12 @@ meta_prefs_get_reduced_resources (void)
   return reduced_resources;
 }
 
+gboolean
+meta_prefs_get_gnome_accessibility ()
+{
+  return gnome_accessibility;
+}
+
 MetaKeyBindingAction
 meta_prefs_get_keybinding_action (const char *name)
 {
